#from asyncio import subprocess
import subprocess
from module.shell import Shell
import os
import sys
from pathlib import Path

class color:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    END = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


class Conn(Shell):

    result = ""

    def __init__(self, config):
        #self.shell_string = '' # this is what our app will use to prefix any commands we send to the database on the shell 
        
        self.config = config # not used right now as i'm just dealing with local docker


        #path = os.path.dirname(os.path.realpath(__file__)) + '/keys/id_rsa'
        path = os.path.join( "module", "connectors", "keys", "id_rsa" )


        self.path_cert = Path(path)
        print('SOMETHING')

        self.ssh_user = 'starbuck'



        if os.path.exists(self.path_cert) == False:
            raise ValueError(f'{color.FAIL}File does not exist: {self.path_cert}{color.END}')

        print(f'{color.HEADER}Found key file: {self.path_cert}{color.END}')
        
        #self.result = self.run_ssh_command(command)

        #self._create_certs()
        
        self.build_shell_string(self.result)


    def build_shell_string(self, ssh_result):
        self.result = ssh_result


        

    def run(self, command):

        # the host key contains a list of hosts
        for host in self.config['database_config']['host']:

            # -o options, including connection timeout in seconds
            # -oStrictHostKeyChecking gets around any invalid root CAs
            # -tt force psueo tty (honestly can't remember why this is here, test without?)
            # -i private key (identity file)
            # -n don't pass standard input, so it doesnt wait for a password and cause 9the command to hang
            # -q supress warnings, -qq suppress fatal too!

            my_command = command.replace("cqlsh", f'cqlsh {host}')
            print(f'{color.HEADER}Connecting to: {self.ssh_user}@{host}{color.END}') 

            try:
                ssh = subprocess.Popen(['ssh',
                                            '-oConnectTimeout=1',
                                            '-oStrictHostKeyChecking=no',
                                            '-oBatchMode=yes',
                                            '-tt',
                                            f'{self.ssh_user}@{host}',
                                            '-i',
                                            f'{self.path_cert}',
                                            f'{my_command}'],
                                        stdin=subprocess.PIPE, 
                                        stdout=subprocess.PIPE,
                                        stderr=subprocess.PIPE) 
                (stdout, stderr) = ssh.communicate()


                if stdout:
                    print(f'{color.HEADER}Running command:{color.END} {my_command}') 
                    print(f'{color.OKGREEN}{stdout.decode("utf-8").strip()}{color.END}')
                    print(f'{color.HEADER}Connection closed{color.END}')
                    return stdout.decode("utf-8").strip(), None
                    
                if (ssh.returncode != 0):
                    print(f'{color.FAIL}{stderr.decode("utf-8").strip()}{color.END}')
                    return None, stderr.decode("utf-8").strip()
                    
                

            except OSError as e:
                print ("OSError > "),e.errno
                print ("OSError > "),e.strerror
                print ("OSError > "),e.filename
                raise ValueError(f'{color.FAIL}OSError{color.END}')

            except:
                print ("Error > "),sys.exc_info()[0]
                raise ValueError(f'{color.FAIL}Error{color.END}')



    def _create_certs(self):

        out, err = Shell.run(f'ssh-keygen -b 2048 -t rsa -f "{self.path_cert}" -q -N ""')
        print(out)
        print(err)
        #Shell.run('ssh-keygen')


from distutils.log import error
import errno
from sre_compile import isstring
import yaml, importlib
from module.connectors import *
import toml
from module.util.console import Color

    
class Connection:

    config_fullpath = "config.toml"
    conn = object
    config = object
    connection_class = object

    def __init__(self):
        config = self._load_config()
        print(config)
        # now lets call a class that matches the config
        if 'environment' not in config['database']:
            raise ValueError(f"{Color.FAIL}Config: error - 'environment' is missing{Color.END}") 
        
        print(f"{Color.OKCYAN}Config: using config setings for {Color.BOLD}'{config['database']['environment']}'{Color.END}")

        try:
            self.connection_class = importlib.import_module(f"module.connectors.{config['database']['environment']}")
        except:
            raise ValueError(f"{Color.FAIL}Connection: could not find a connection module called '{config['database']['environment']}'{Color.END}")
        

        #my_class = locate(f"module.connection.{config['database_env']}")

        self.config = self._parse_config(config)


        
        self.db_env = '' # filed in from config


        
    

    def run(self, command):
        """
        Attempts to run a command against the database

        :param input: the cqlsh result to parse and format
        :return: the result of the command on the shell
        """
        c = self.connection_class.Conn(self.config) 
        
        return c.run(command)

    def _load_config(self):

        result = {}

        print(f"{Color.HEADER}Config: attempting to load config file...'{Color.END}")

        # LOAD the config file
        
        f = toml.load(self.config_fullpath) 

       
        print(f"{Color.OKCYAN}Config: taml file parsed...'{Color.END}")
        return f


    def _parse_config(self, config):
        # future method to make sure certain config keys exist like host and env

        # make sure there's a 'host' in there some where
        if 'host' not in config['database']:
            print(f"{Color.FAIL}Config: no 'host' list found in the config file{Color.END}")
            exit()
        
        host = config['database']['host']

        # ok, so we know it exists, we're going to see if it's a string or a list. If it's a string, convert it to a list with 1 item
        if isstring(host):
            temp_list = []
            temp_list.append(host)

            config['database']['host'] = temp_list
        
        print(f"{Color.OKCYAN}Config: using the following hosts: {config['database']['host']}{Color.END}")
        return config


from setuptools import setup, find_packages

requires = [
    'psycopg2',
    'jsonschema'
]

setup(
    name='ivypy',
    version='0.1',
    description='Ivy for Python',
    author='James Randell',
    author_email='jamesrandell@me.com',
    keywords='Ivy Python',
    packages=find_packages(),
    include_package_data=True,
    install_requires=requires
)


from .util import Console
from .validation import Validation

from importlib import import_module

from .dictionary import Dictionary

class Datasource(Dictionary, Validation):

    queryParts:dict = {
        'field':{},
        'pagination':{},
        'where':{},
        'order':{},
        'action':'',
        'table': '',
        'database': '',
        'pk':{},
        'data':{}
    }
    
    @property
    def data(self):
        return self._data


    @data.setter
    def data(self, dataDict):
        dataList: list = [*dataDict]
        self.queryParts["field"] = Validation.fieldExists(self, dataList)
        
        dataDict, error = Validation.check(self, dataDict)

        
        self.error = error

        for field in self.queryParts["field"]:
            self._data[field] = dataDict[field]
            self.queryParts['data'][field] = dataDict[field]


    def __init__(self, schemaFile):
        Console.info('Datasource instance.................')
        super().__init__(schemaFile)
        
        database_type = self.database_spec['type']

        self.db = import_module(f'.connection.{database_type}', package='ivyorm').Connection()
        self._data = {}
        self.error = {}
        self.id: any

        self.database(self.database_spec['name'])
        self.table(self.table_spec['name'])
        self.pk(self.table_spec['pk'])
        

    def select(self):
        self.queryParts['action'] = 'select'

        if not self.error:

            success, result = self.db.query(self.queryParts)
            self._data = result
            return success
            

        return False


    def insert(self, data: dict = None):
        self.queryParts['action'] = 'insert'

        if data:
            self.data = data

        if not self.error:
            success, result = self.db.query(self.queryParts)
            self.id = result[0][self.queryParts['pk'][0].lower()]
            self.data[ self.queryParts['pk'][0] ] = self.id
            return success

        return False


    def field(self, fields: list):
        Console.info('field')
        self.queryParts["field"] = Validation.fieldExists(self, fields)
        return self
    

    def order(self, fields):
        Console.info('order')
        return self


    def where(self, fields):
        Console.info('where')
        return self


    def limit(self, fields):
        Console.info('limit')
        return self    


    def database(self, database: str):
        self.queryParts["database"] = database


    def table(self, table: str):
        self.queryParts["table"] = table


    def pk(self, pkList: list):
        self.queryParts['pk'] = pkList


    def object(self, object):
        self.table(object)


    def create(self):
        self.queryParts["action"] = 'create'

        fields = {}
        for field in self.field_spec:
            fields[field] = self.field_spec[field]['back']

        self.queryParts["field"] = fields
        self.db.query(self.queryParts)


    def drop(self):
        self.queryParts["action"] = 'drop'
        self.db.query(self.queryParts)